<?php

namespace App\Http\Controllers\Api\V1;

use App\Models\BaseSettingsModel;
use App\Models\DreamInfoModel;
use App\Models\SearchInfoModel;
use App\Models\UserCareDream;
use App\Models\UserCareUser;
use App\Models\UserDream;
use App\Models\UserInfoModel;
use Illuminate\Http\Request;
use App\Services\Base\ErrorCode;
class IndexController extends Controller
{

    /**
     * @api {get} /api/index/index 首页
     * @apiDescription 首页
     * @apiGroup Index
     * @apiParam {string}  type   (热门)hot/(潮流)trend/(最新)news
     * @apiParam {int}      [page=1]            页码（分页参数）
     * @apiPermission none
     * @apiVersion 0.1.0
     * @apiSuccessExample {json} Success-Response:
     * HTTP/1.1 200 OK
     *{
     *   "status": true,
     *  "status_code": 0,
     *   "message": "",
     *   "data": {
     *       "banner": [            轮播图
     *          {
     *              "value": "http://w17.9026.com/img/banner/banner_2.png",
     *          },
     *          {
     *               "value": "http://w17.9026.com/img/banner/banner_3.png",
     *           },
     *      ],
     *   "users": {
     *      "1": [               1用户ID
     *          [
     *              {                                   用户梦想互动
     *                  "id": 8,
     *                  "dream_id": 1,
     *                  "title": "2",
     *                  "pic1": "www.buniang.com",
     *                 "pic2": "",
     *                  "pic3": "",
     *                 "pic4": "",
     *                  "pic5": "",
     *                 "pic6": "",
     *                  "pic7": "",
     *                   "pic8": "",
     *                  "pic9": "",
     *                  "status": 1,
     *                  "created_at": "2017-06-23 08:13:39",
     *                  "updated_at": "2017-06-23 08:13:39",
     *                  "deleted_at": null,
     *                  "user_id": 1
     *              },
     *              {
     *                  "id": 9,
     *                  "dream_id": 1,
     *                  "title": "2haha",
     *                  "pic1": "www.buniang.com",
     *                  "pic2": "",
     *                  "pic3": "",
     *                  "pic4": "",
     *                  "pic5": "",
     *                  "pic6": "",
     *                  "pic7": "",
     *                  "pic8": "",
     *                  "pic9": "",
     *                  "status": 1,
     *                  "created_at": "2017-06-23 08:15:01",
     *                  "updated_at": "2017-06-23 08:15:01",
     *                 "deleted_at": null,
     *                  "user_id": 1
     *              },
     *          ],
     *          "http://www.wsfjq.com/photos/bd119684755.jpg"            用户头像
     *           3                                      新消息条数
     *      ],
     *}
     *      "dreams": {                             梦想列表
     *      "current_page": 1,
     *      "data": [
     *           {
     *              "id": 1,
     *               "user_id": 1,
     *              "dream": "梦想标题",
     *               "about": "梦想烧",
     *               "video": "",
     *               "money": 10000,
     *              "time": 720000,
     *              "get_money": 0,
     *              "status": 0,
     *              "updated_at": "2017-06-22 09:42:41",
     *              "dream_user": {                         梦想所有者
     *               "nickname": "ha",
     *              "pic": "http://www.wsfjq.com/photos/bd119684755.jpg",
     *               "money": 123568,
     *               "sex": 1,
     *               "signture": "signture",
     *              "city": "",
     *              "birthday": "2017-06-21",
     *               "status": 1,
     *           },
     *          "care_num": 1,                        关注人数
     *          "dream_imgs_first": {                 梦想封面图片
     *              "title": "",
     *               "pic": "http://w17.9026.com/img/banner/banner_3.png"
     *          },
     *          "dream_detailed": {                    点击进入梦想详情
     *              "dream_imgs": [
     *                  {
     *                      "title": "",
     *                      "pic": "http://w17.9026.com/img/banner/banner_3.png"
     *                  },
     *                  {
     *                      "title": "",
     *                     "pic": "http://w17.9026.com/img/banner/banner_3.png"
     *                  }
     *              ],
     *              "comments": [
     *                   {
     *                      "user_id": 2,
     *                      "level": 0,
     *                      "content": "EST评论",
     *                      "updated_at": null,
     *                      "pic": "http://www.wsfjq.com/photos/bd119684755.jpg",     评论者头像
     *                     "replay": [
     *                         {
     *                              "user_id": 2,
     *                              "level": 0,
     *                              "content": "拉了",
     *                              "updated_at": null,
     *                              "pic": "http://www.wsfjq.com/photos/bd119684755.jpg"         回复者头像
     *                          },
     *                          {
     *                              "user_id": 3,
     *                              "level": 0,
     *                              "content": "userid3回复了你的消息",
     *                              "updated_at": null,
     *                              "pic": "http://www.wsfjq.com/photos/bd119684755.jpg"
     *                          }
     *                      ]
     *                  },
     *         }
     *      ]
     *      },
     *      "from": 1,
     *      "last_page": 1,
     *      "next_page_url": null,
     *      "path": "http://www.miao.com/api/index/index",
     *      "per_page": 20,
     *       "prev_page_url": null,
     *      "to": 8,
     *      "total": 8
     *     }
     *   }
     *}
     * @apiErrorExample {json} Error-Response:
     * HTTP/1.1 400 Bad Request
     * {
     *     "state": false,
     *     "code": 1000,
     *     "message": "传入参数不正确",
     *     "data": null or []
     * }
     */
    public function index(Request $request)
    {
        $user = $this->getUser();
        //        关注的梦想
        $users =  $user->UserCareDream;
//        $arr = UserInfoModel::where('id',$user->id)->whereHas('UserCareDream', function ($query) {
//            $query->where('interaction_number', '>', 0);
//        })->get();
//        dd($arr->toArray());
//        $users = [] ;
//        $arr1 = [];
//        foreach ($arr as $k => $v){
//            if ($v->pivot->interaction_number > 0) {
//                if (!array_key_exists($v->pivot->dream_user_id,$arr1)) {
//                    $arr1[$v->pivot->dream_user_id] = $v->pivot->interaction_number;
//                }else{
//                    $arr1[$v->pivot->dream_user_id] += $v->pivot->interaction_number;
//                }
//            }
//        }
//        dd($arr1);
//        foreach ($arr1 as $k => $v){
//            $a = UserInfoModel::find($k);
//            $users[$k] =[$a->allInteraction,$a->pic,$v] ;
//        }
        $type = $request->type;
        if ($type == 'trend') {
            $dreams = DreamInfoModel::orderBy('score','desc')->offset(20)->limit(100)->paginate(20);
            $this->dreams($dreams);
            return $this->api(compact('users','dreams'));
        } elseif ($type == 'news') {
            $dreams = DreamInfoModel::orderBy('score','desc')->offset(100)->limit(500)->paginate(20);
            $this->dreams($dreams);
            return $this->api(compact('users','dreams'));
        } else{
            $banners = $this->getBanner();
    //        获取其他用户信息 及梦想
            $dreams = DreamInfoModel::orderBy('score','desc')->limit(20)->paginate(20);
            $this->dreams($dreams);
            return $this->api(compact('banners','users','dreams'));
        }
    }


    /**
     * @api {get}  /api/index/search 搜索(search)
     * @apiDescription 搜索(search)
     * @apiGroup Index
     * @apiPermission none
     * @apiVersion 0.1.0
     * @apiParam {string}  keyword   关键字可选  (ha)
     * @apiSuccessExample {json} Success-Response:
     * HTTP/1.1 200 OK
     *     get
     *{
     *    "status": true,
     *  "status_code": 0,
     *  "message": "",
     *  "data": {
     *      "arr": {    // 热门搜索
     *       ...
     *      },
     *      "data1": [
     *          {
     *              "search": "ha",  //  历史搜索
     *          }
     *      ]
     *  }
     *}
     * post
     * {
     *   "status": true,
     *   "status_code": 0,
     *   "message": "",
     *   "data": {
     *      "data1": [
     *          ...  //用户
     *       ],
     *       "data2": [
     *            ...   //梦想
     *       ],
     *      "data3": [
     *            ...   //标签
     *       ]
     *    }
     *}
     * @apiErrorExample {json} Error-Response:
     * HTTP/1.1 400 Bad Request
     */
    public function search(Request $request)
    {
        $user = $this->getUser();
        $keyword ='%'.$request->keyword.'%';
        $data1 = UserInfoModel::where('nickname','like',$keyword)->get();
        $data2 = DreamInfoModel::where('dream','like',$keyword)->
            orWhere('sign','like',$keyword)->get();
        $data3  = BaseSettingsModel::where('category','sign')->where('value','like',$keyword)->get();
        if (empty($request->keyword)) {
//            历史搜索
                $data1  = $user->search()->orderBy('id','desc')->limit(10)->get();
//            热门搜索
                $data2 = SearchInfoModel::get();
                $arr = [];
                foreach ($data2 as $k => $v) {
                    if (count($arr) == 8) {
                        break;
                    }
                    if (!array_key_exists($v->search,$arr)) {
                        $arr[$v->search] = $v->times;
                    }else{
                        $arr[$v->search] += $v->times;
                    }
                }
                arsort($arr);
                return $this->api(compact('arr','data1'));
        }
//        写入搜索记录
        $this->insertSearchTable($user->id,$request->keyword);
        return $this->api(compact('data1','data2','data3'));
    }

    /**
     * @api {get}  /api/index/user_search 用户搜索(search)
     * @apiDescription 用户搜索(search)
     * @apiGroup Index
     * @apiPermission none
     * @apiVersion 0.1.0
     * @apiParam {string}  keyword   关键字
     * @apiSuccessExample {json} Success-Response:
     * HTTP/1.1 200 OK
     *{
     *    "status": true,
     *   "status_code": 0,
     *   "message": "",
     *  "data": {
     *       "data1": [
     *          "nickname": "ha",  昵称
     *          "pic": "",    头像
     *             ...
     *       ]
     *   }
     *}
     * @apiErrorExample {json} Error-Response:
     * HTTP/1.1 400 Bad Request
     */
    public function userSearch(Request $request)
    {
        $user = $this->getUser();
        if (empty($request->keyword)) {
            return $this->api('');
        }
        $keyword ='%'.$request->keyword.'%';
        $data1 = UserInfoModel::where('nickname','like',$keyword)->get();
        $this->insertSearchTable($user->id,$request->keyword);
        return $this->api(compact('data1'));
    }



    //获取轮播图
    public function getBanner()
    {
        $banner =  BaseSettingsModel::where(['category' => 'banner'])->where(['status' => '1'])
            ->orderBy('sort')->get()->toArray();
        return $banner;
    }

//    完善梦想信息
    public function dreams($dreams)
    {
        foreach ($dreams as $k => $dream) {
            $dream->dream_user =  $dream->user;
//            计算被关注总人数
            $user_id =  $dream->user_id;
            $data = UserCareDream::where('dream_id',$dream->id)->get();
            $dream->care_num = count($data);
            $dream->dream_imgs_first = $dream->dreamImgsFirst?$dream->dreamImgsFirst->pic:'';
//            梦想详情  detailed  图片 ，评论 ，回复
            $arr = [] ;
            $arr['dream_imgs'] = $dream->imgs;
            $dream->dream_detailed = $arr;
        }
    }
//    查看关注用户的最新动态
    /**
     * @api {get}  /api/index/news_info 关注用户动态详情
     * @apiDescription 关注用户动态详情
     * @apiGroup Index
     * @apiPermission none
     * @apiVersion 0.1.0
     * @apiParam {int}  id   被点击用户ID
     * @apiSuccessExample {json} Success-Response:
     * HTTP/1.1 200 OK
     *{
     *   "status": true,
     *   "status_code": 0,
     *   "message": "",
     *  "data": {
     *      "all_interaction": [                          互动
     *          {
     *              "id": 7,
     *              "dream_id": 2,
     *              "title": "梦想动态有力来了",
     *              "pic1": "http\"//www.baidu.com",
     *              "pic2": "",
     *               "pic3": "",
     *              "pic4": "",
     *              "pic5": "",
     *              "pic6": "",
     *              "pic7": "",
     *              "pic8": "",
     *              "pic9": "",
     *              "status": 1,
     *              "created_at": "2017-06-22 21:09:18",
     *              "updated_at": "2017-06-22 21:09:18",
     *              "deleted_at": null,
     *              "user_id": 2,
     *              "time": 720000,                           倒计时
     *              "comments": [                            评论
     *                  {
     *                      "interaction_id": 7,
     *                      "user_id": 1,
     *                      "level": 0,
     *                      "content": "不错的梦想",
     *                      "updated_at": null,
     *                      "pic": "http://www.wsfjq.com/photos/bd119684755.jpg",
     *                      "replay": [                       回复
     *                          {
     *                              "user_id": 1,
     *                              "level": 0,
     *                              "content": "我也觉得不错",
     *                              "updated_at": null,
     *                              "pic": "http://www.wsfjq.com/photos/bd119684755.jpg"
     *                          },
     *                          {
     *                              "user_id": 6,
     *                              "level": 0,
     *                              "content": "祝你梦想成真",
     *                              "updated_at": null,
     *                              "pic": "http://www.wsfjq.com/photos/bd119684755.jpg"
     *                          }
     *                      ]
     *                  }
     *               ]
     *          },
     *      ]
     *  }
     *}
     * @apiErrorExample {json} Error-Response:
     * HTTP/1.1 400 Bad Request
     */
    public function newsInfo(Request $request)
    {
//        查看后user_care_user dream_info 更新为零 首页不再显示
        $user = $this->getUser();
        $other_id = $request->id;
        if (empty($other_id))  return $this->error(ErrorCode::MEMBER_NOT_EXIST);
        UserCareDream::where('user_id',$user->id)->where('dream_user_id',$other_id)->update(['interaction_number'=>0]);
        $data = UserInfoModel::where('id',$other_id)->with(['allInteraction'=>function ($query){
            $query->orderBy('id','desc');
        }])->first();
//        $data = UserInfoModel::find($other_id)->allInteraction;
//        dd($data) ;
//        dd($data->allInteraction);
        foreach ($data->allInteraction as $item) {
            $item->time = DreamInfoModel::find($item->dream_id)->time;
            $item->comments = $item->comments;
            foreach ($item->comments as $k => $v) {
                $v->pic = UserInfoModel::find($v->user_id)->pic;
                $v->replay = $v->replay;
                foreach ($v->replay as $k1 => $v1) {
                    $v1->pic = UserInfoModel::find($v1->user_id)->pic;
                }
            }
        }
        return $this->api($data);
    }
}
